#!/usr/bin/env python
import ast
import dis
import sys
import types
from astoptimizer import Config, parse_ast, optimize_ast, compile_ast

def dump_bytecode(code):
    dis.dis(code)
    for const in code.co_consts:
        if not isinstance(const, types.CodeType):
            continue
        print("")
        print("%s:" % const)
        dump_bytecode(const)

def main():
    if len(sys.argv) < 2:
        print("usage: %s code ..." % sys.argv[0])
        print("   or: %s -   # read input from stdin" % sys.argv[0])
        print("example: %s '1+1'" % sys.argv[0])
        sys.exit(1)
    if len(sys.argv) == 2 and sys.argv[1] == '-':
        code_str = sys.stdin.read()
    else:
        code_str = '\n'.join(sys.argv[1:])

    config = Config('builtin_funcs', 'pythonenv')
    config.use_experimental_vars = True
    print("Config features: %s" % ', '.join(sorted(config.features)))
    print("")

    tree = parse_ast(code_str)
    print("Original AST:")
    print(ast.dump(tree))
    print("")

    print("Original bytecode:")
    try:
        code = compile(code_str, "<string>", "exec")
    except SyntaxError:
        print("*** SYNTAX ERROR ***")
    else:
        dump_bytecode(code)
    print("")

    tree = optimize_ast(tree, config)
    print("Optimised AST:")
    print(ast.dump(tree))
    print("")

    print("Optimized bytecode:")
    try:
        code = compile_ast(tree)
    except SyntaxError:
        print("*** SYNTAX ERROR ***")
    else:
        dump_bytecode(code)
        print("")

        print("Execute code:")
        exec(code)

if __name__ == "__main__":
    main()

